--the basics!
local L = AceLibrary("AceLocale-2.2"):new("Voyeur")
Voyeur = AceLibrary("AceAddon-2.0"):new("AceHook-2.1","AceConsole-2.0","AceDB-2.0")

Voyeur.version = tonumber(string.sub("$Rev: 48394 $", 12, -3))
-- Voyeur.version = "r" .. Voyeur.revision
Voyeur.date = string.sub("$Date: 2007-09-07 19:04:49 -0400 (Fri, 07 Sep 2007) $", 8, 17)
Voyeur.options = AceLibrary("AceConsole-2.0"):InjectAceOptionsTable(Voyeur, {
	type = 'group',
	args = {},
})

BINDING_HEADER_VOYEUR = L["Voyeur"]
BINDING_NAME_VOYEUR = L["Inspect Target"]

local inspectCache = {}
local updateInspect = true

function Voyeur:OnInitialize()
	self:RegisterDB("VoyeurDB")
	self:RegisterChatCommand({"/voyeur"}, self.options)

	InspectFrame_LoadUI()
end

function Voyeur:OnEnable()
	self:UpdateConsoleOptions()
	self:Hook("InspectFrame_OnUpdate", true)
	self:Hook("InspectFrame_Show", true)
	self:Hook("InspectFrame_UnitChanged", true)
	self:Hook("InspectFrame_OnShow", true)
	self:Hook("InspectFrame_OnEvent", true)

	self:Hook("InspectPaperDollItemSlotButton_Update", true)
	self:Hook("InspectPaperDollFrame_OnShow", true)
	
	self:Hook("InspectPaperDollItemSlotButton_OnEvent", true)
	self:Hook("InspectPaperDollItemSlotButton_OnClick", true)	
	self:Hook("InspectPaperDollItemSlotButton_OnEnter", true)
	
	self:UpdateMovable()

-- Prevent the Player Menu inspect function from greying out beyond inspect distance
	UnitPopupButtons["INSPECT"] = { text = TEXT(INSPECT), dist = 0 }
end

function Voyeur:UpdateMovable(v)
	if not v then
		v =	self.db.profile.move
	end
	InspectFrame:SetMovable(v)
	local visible = InspectFrame:IsVisible()
	HideUIPanel(InspectFrame)
	if v then
		UIPanelWindows["InspectFrame"] = nil
		if not self:IsHooked(InspectFrame, "OnMouseDown") then
			self:HookScript(InspectFrame, "OnMouseDown", function() this:StartMoving() end)
		end
		if not self:IsHooked(InspectFrame, "OnMouseUp") then
			self:HookScript(InspectFrame, "OnMouseUp", function() this:StopMovingOrSizing() end)
		end
	else
		UIPanelWindows["InspectFrame"] = { area = "left", pushable = 0 }
		if self:IsHooked(InspectFrame, "OnMouseDown") then
			self:Unhook(InspectFrame, "OnMouseDown")
		end
		if self:IsHooked(InspectFrame, "OnMouseUp") then
			self:Unhook(InspectFrame, "OnMouseUp")
		end
	end
	if visible then
		ShowUIPanel(InspectFrame)
	end
end

function Voyeur:OnDisable()
	UIPanelWindows["InspectFrame"] = { area = "left", pushable = 0 }
	InspectFrame:SetMovable(false)

	UnitPopupButtons["INSPECT"] = { text = TEXT(INSPECT), dist = 1 }
end
-- Let's look at the other faction, even though we can't determine their gear.
function Voyeur:InspectFrame_Show(unit)
	HideUIPanel(InspectFrame)
	if (UnitExists(unit) and UnitIsPlayer(unit)) then
		InspectFrame.unit = unit
		self:InspectFrame_UnitChanged()
		if not InspectFrame:IsVisible() then
			ShowUIPanel(InspectFrame)
		end
	else
		updateInspect = true
	end
end

function VoyeurInspect()
	if InspectFrame:IsVisible() then
		HideUIPanel(InspectFrame)
	else
		if UnitExists("target") then
			return InspectUnit("target")
		elseif InspectFrame.player then
			return ShowUIPanel(InspectFrame)
		end
	end
end

function Voyeur:InspectFrame_OnEvent(event)
	local unit = InspectFrame.unit
	if not InspectFrame:IsVisible() then
		return
	end
	if (event == "PLAYER_TARGET_CHANGED") and (unit == "target") then
		if UnitExists(unit) and UnitIsPlayer(unit) then
			updateInspect = true
			self:InspectFrame_UnitChanged()
		end
		return
	elseif (event == "UNIT_PORTRAIT_UPDATE") then
		if (arg1 == unit) then
			SetPortraitTexture(InspectFramePortrait, arg1)
		end
		return
	elseif (event == "UNIT_NAME_UPDATE") then
		if (arg1 == unit) then
			local guildname, guildtitle, guildrank = GetGuildInfo(arg1)
			InspectNameText:SetText(UnitName(arg1))
		end
		return
	end
end

function Voyeur:InspectFrame_UnitChanged()
	local unit = InspectFrame.unit
	NotifyInspect(unit)
	SetPortraitTexture(InspectFramePortrait, unit)
	if (InspectFrame.player ~= UnitName(unit)) then
		updateInspect = true
	end
	if (updateInspect == true) then
		InspectFrame.player = UnitName(unit)
		inspectCache = {}
		for i=1,19 do
			inspectCache[i] = GetInventoryItemLink(unit,i)
		end
		InspectPaperDollFrame_OnShow()
	end
	if CanInspect(unit) then
		updateInspect = false
	else
		updateInspect = true
	end
	InspectNameText:SetWidth(300)
	InspectNameText:SetText(UnitPVPName(unit))
	local guildname, guildtitle, guildrank = GetGuildInfo(unit)
	if(guildname ~= nil and guildtitle ~= nil and guildrank ~= nil) then
		InspectTitleText:SetText(format(TEXT(GUILD_TITLE_TEMPLATE), guildtitle, guildname))
		InspectTitleText:Show()
	else
		InspectTitleText:Hide()
	end
end

function Voyeur:InspectFrame_OnUpdate()
	local unit = InspectFrame.unit
	if UnitExists(unit) and CanInspect(unit) and (updateInspect == true) then
		self:InspectFrame_UnitChanged()
	elseif not UnitExists(unit) or not CanInspect(unit) then
		updateInspect = true
	end
end

function Voyeur:InspectPaperDollItemSlotButton_OnEvent(event)
	if ( event == "UNIT_INVENTORY_CHANGED" ) then
		if ( arg1 == InspectFrame.unit ) then
			local id = this:GetID()
			inspectCache[id] = GetInventoryItemLink(InspectFrame.unit ,id)
			InspectPaperDollItemSlotButton_Update(this)
		end
		return
	end
end

function Voyeur:InspectPaperDollFrame_OnShow()
	local unit = InspectFrame.unit
	if not UnitExists(unit) then return end
	InspectModelFrame:SetUnit(unit)
	InspectPaperDollFrame_SetLevel()
	InspectPaperDollItemSlotButton_Update(InspectHeadSlot)
	InspectPaperDollItemSlotButton_Update(InspectNeckSlot)
	InspectPaperDollItemSlotButton_Update(InspectShoulderSlot)
	InspectPaperDollItemSlotButton_Update(InspectBackSlot)
	InspectPaperDollItemSlotButton_Update(InspectChestSlot)
	InspectPaperDollItemSlotButton_Update(InspectShirtSlot)
	InspectPaperDollItemSlotButton_Update(InspectTabardSlot)
	InspectPaperDollItemSlotButton_Update(InspectWristSlot)
	InspectPaperDollItemSlotButton_Update(InspectHandsSlot)
	InspectPaperDollItemSlotButton_Update(InspectWaistSlot)
	InspectPaperDollItemSlotButton_Update(InspectLegsSlot)
	InspectPaperDollItemSlotButton_Update(InspectFeetSlot)
	InspectPaperDollItemSlotButton_Update(InspectFinger0Slot)
	InspectPaperDollItemSlotButton_Update(InspectFinger1Slot)
	InspectPaperDollItemSlotButton_Update(InspectTrinket0Slot)
	InspectPaperDollItemSlotButton_Update(InspectTrinket1Slot)
	InspectPaperDollItemSlotButton_Update(InspectMainHandSlot)
	InspectPaperDollItemSlotButton_Update(InspectSecondaryHandSlot)
	InspectPaperDollItemSlotButton_Update(InspectRangedSlot)
end

function Voyeur:InspectPaperDollItemSlotButton_Update(button)
	local unit = InspectFrame.unit
	local itemString = self:GetInspectItemLink(button:GetID())
	local _, _, _, _, _, _, _, _, _, textureName = GetItemInfo(itemString or "")
	if textureName then
		SetItemButtonTexture(button, textureName)
		button.hasItem = 1
	else
		local textureName = button.backgroundTextureName
		if ( button.checkRelic and UnitHasRelicSlot(unit) ) then
			textureName = "Interface\\Paperdoll\\UI-PaperDoll-Slot-Relic.blp"
		end
		SetItemButtonTexture(button, textureName)
		SetItemButtonCount(button, 0)
		button.hasItem = nil
	end
end

function Voyeur:InspectFrame_OnShow()
	local unit = InspectFrame.unit
	if not UnitExists(unit) then return end
	PlaySound("igCharacterInfoOpen")
	SetPortraitTexture(InspectFramePortrait, unit)
	InspectNameText:SetText(UnitName(unit))
end

function Voyeur:InspectPaperDollItemSlotButton_OnClick(button)
	if IsControlKeyDown() then
		DressUpItemLink(self:GetInspectItemLink(this:GetID()))
	elseif IsShiftKeyDown() then
		ChatEdit_InsertLink(self:GetInspectItemLink(this:GetID()))
	end
end

function Voyeur:InspectPaperDollItemSlotButton_OnEnter()
	local itemString = self:GetInspectItemLink(this:GetID())
	GameTooltip:SetOwner(this, "ANCHOR_RIGHT")
	if itemString then
		GameTooltip:SetHyperlink(itemString)
		if ( IsShiftKeyDown() ) then
			GameTooltip_ShowCompareItem()
		end
	else
		local text = getglobal(strupper(strsub(this:GetName(), 8)))
		if (this.checkRelic and UnitHasRelicSlot(InspectFrame.unit)) then
			text = getglobal("RELICSLOT")
		end
		GameTooltip:SetText(text)
	end
	CursorUpdate()
end

function Voyeur:GetInspectItemLink(id)
	if type(id) == "string" then
		id = GetInventorySlotInfo(id)
	end
	if not id or type(id) ~= "number" then return end
	return inspectCache[id]
end

function Voyeur:UpdateConsoleOptions()
	local optionArgs = self.options.args
	optionArgs.move = {
		type = 'toggle',
		name = "Movable Inspect Frame", 
		desc = "Movable Inspect Frame",
		get = function()
			return self.db.profile.move or false
		end,
		set = function(v)
			self.db.profile.move = v
			self:UpdateMovable(v)
		end,
	}
end